                        zeusemulate "nameless"

; This is as much of the OS as this game requires to run, don't try to use this for anything else.
;
; The code "si.bin" is a hand-assembled space invaders game from 1978 or so...
;

; VDU equates

VDU_Base                equ $0C00                       ; Somewhere for the text based display
nChars                  equ 64                          ; Screen width
nLines                  equ 25                          ; Screen height
VDU_Size                equ nChars*nLines               ; Size

; I/O

KbdPort                 equ $FF                         ; Dummy keyboard port

; VDU character codes

cCR                     equ $81                         ; CR
cLF                     equ $82                         ; LF
cCLS                    equ $84                         ; CLS
cBS                     equ $85                         ; BS

; Start of OS

                        org $0000                       ;
                        ld sp,#800                      ;
                        di                              ;
                        jp Start                        ;

                        org $8                          ; Print StrZ
                        jp Rst_08                       ;

                        org $10                         ;
                        jp Rst_10                       ;

                        org $18                         ; Read keyboard
                        jp Rst_18                       ;

                        org $20                         ;
                        jp Rst_20                       ;

                        org $28                         ;
                        jp Rst_28                       ;

                        org $30                         ;
                        jp Rst_30                       ;

                        org $38                         ;
                        jp Rst_38                       ;

; Start the code

Start                   ld a,$00
                        ld i,a

                        call Init_Display               ; Clear it
                        jp $C000                        ; Start the code

; This prints the stringz following it

Rst_08                  ex (sp),hl                      ; Get a pointer to the data

Rst08_Lp                ld a,(hl)                       ; Get the next char
                        inc hl                          ; Skip

                        or a                            ; Zero?
                        jr z Rst08_X                    ; Yes, we're done

                        call Print                      ; Print the character

                        jp Rst08_Lp                     ; Loop

                        ; Done, exit

Rst08_X                 ex (sp),hl                      ; Setup the return address
                        ret                             ; Done

; This reads the keyboard and returns with the key pressed in A (01 = A, etc)
; It flushes and ignores any keys that are waiting in the buffer when first called

Rst_18                  in a,(KbdPort)                  ; Read a dummy port, let the emulator handle keys
                        or a                            ; 0?
                        jr nz Rst_18                    ; No, ignore it

                        ; We'll take the next keypress

Rst_18_Lp2              in a,(KbdPort)                  ; Read a dummy port, let the emulator handle keys
                        or a                            ; 0?
                        jr z Rst_18_Lp2                 ; Yes, wait for a key-press

                        ret                             ; Done, return the key

; Unimplemented restarts

Rst_10                  jp Rst_10                       ;
Rst_20                  jp Rst_20                       ;
Rst_28                  jp Rst_28                       ;
Rst_30                  jp Rst_30                       ;

; We'll just ignore these

Rst_38                  ret                             ;

; Init the display

Init_Display            call CLS                        ; Clear the screen
                        ret                             ; Done

; Print the character in A

Print                   push hl,de,bc,af                ; Save

                        cp cCR                          ; CR?
                        jr z Print_CR                   ;

                        cp cLF                          ; LF?
                        jr z Print_LF                   ;

                        cp cBS                          ; BS?
                        jr z Print_BS                   ;

                        cp cCLS                         ; CLS?
                        jr z Print_CLS                  ;

                        ; Assume it's printable

                        push af                         ; Save the char
                        call Calc_pCur                  ; Calc the cursor address
                        pop af                          ;

                        ld (hl),a                       ; Set the character

                        ; Step the cursor

                        ld a,(Cur_X)                    ; Inc it
                        inc a                           ;
                        ld (Cur_X),a                    ;

                        cp nChars                       ; Hit RHS?
                        jr nz Print_X                   ; No, done

                        ; We wrap

                        ld a,0                          ;
                        ld (Cur_X),a                    ;

                        ld a,(Cur_Y)                    ; Inc it
                        inc a                           ;
                        ld (Cur_Y),a                    ;

                        cp nLines                       ; Hit bottom?
                        jr nz Print_X                   ; No, done

                        ; We scroll the screen

                        ld hl,VDU_Base+nChars           ;
                        ld de,VDU_Base                  ;
                        ld bc,nChars*(nLines-1)         ;
                        ldir                            ;

                        ld hl,de                        ; Point at the last line
                        ld (hl),0                       ; Clear the first char
                        inc de                          ;
                        ld bc,nChars-1                  ;
                        ldir                            ; Copy across

                        ld a,nLines-1                   ; Rewind Y
                        ld (Cur_Y),a                    ;

                        ; Done

Print_X                 pop af,bc,de,hl                 ; Restore
                        ret                             ;

        ; CR

Print_CR                ld a,0                          ;
                        ld (Cur_X),a                    ;

                        jp Print_X                      ; Done

        ; LF

Print_LF                ld a,(Cur_Y)                    ;
                        inc a                           ;
                        ld (Cur_Y),a                    ;

                        jp Print_X                      ; Done

        ; BS

Print_BS                ld a,(Cur_X)                    ; Move back to start of line
                        or a                            ;
                        jr z Print_X                    ;

                        dec a                           ;
                        ld (Cur_X),a                    ;

                        jp Print_X                      ; Done

        ; CLS

Print_CLS               call CLS                        ;

                        jp Print_X                      ; Done

; Calc the cursor position into HL

Calc_pCur               ld hl,Line_Bases                ; Point at the table
                        ld a,(Cur_Y)                    ; Y
                        add a,a                         ; Y*2
                        ld c,a                          ;
                        ld b,0                          ;
                        add hl,bc                       ;

                        ld a,(hl)                       ;
                        inc hl                          ;
                        ld h,(hl)                       ;
                        ld l,a                          ;

                        ; Now, add the X

                        ld a,(Cur_X)                    ; X
                        ld c,a                          ;
                        add hl,bc                       ;

                        ret                             ; Done

; Clear the VDU

CLS                     ld a,$00                        ; Space is the same
                        ld hl,VDU_Base                  ;
                        ld de,VDU_Base+1                ;
                        ld bc,VDU_Size-1                ;
                        ld (hl),a                       ;
                        ldir                            ;

                        ld a,0                          ;
                        ld (Cur_X),a                    ;
                        ld (Cur_Y),a                    ;
                        ret                             ; Done

; VDU Line starts

Addr                    = VDU_Base                      ;
Line_Bases              loop 25                         ;
                          dw Addr                       ; Start of this line
                          Addr = Addr + nChars          ; Next
                          lend                          ;

; Something unknown is called in here, gawd knows what it is supposed to do. Frame sync, perhaps?

                        while * < $800                  ; We just fill the rest with return instructions
                          ret                           ;
                          wend                          ;

; Make this ROM

                        mem_rom $0000,$400              ; 1K of ROM

; Variables

                        org $800                        ; Somewhere out of the way

Cur_X                   ds 1                            ;
Cur_Y                   ds 1                            ;

; This is where the keyboard buffer lives

                        org $1C00
                        ds 64,$FF

; Get the code

                        import_bin "si.bin",$C000


